#ifndef MYX_REDIS_PARSER_INL_HPP_
#define MYX_REDIS_PARSER_INL_HPP_

#pragma once

#ifndef MYXLIB_HEADER_ONLY
#include <myx/redis/parser.hpp>
#endif

#include <QDebug>

namespace myx {

namespace redis {

MYXLIB_INLINE Parser::Parser( Lexer* lexer, QObject* parent ) :
	QObject( parent ),
	m_lexer( lexer )
{
	connect( m_lexer, &Lexer::character, this, &Parser::readCharacter );
	connect( m_lexer, &Lexer::unsafeString, this, &Parser::readUnsafeString );
	connect( m_lexer, &Lexer::safeString, this, &Parser::readSafeString );
}


MYXLIB_INLINE void Parser::readCharacter( const char c )
{
	switch ( c )
	{
	case '+':
		m_stack.append( Task( Reply::kStatus ) );    break;
	case '-':
		m_stack.append( Task( Reply::kError ) );     break;
	case ':':
		m_stack.append( Task( Reply::kInteger ) );   break;
	case '$':
		m_stack.append( Task( Reply::kBulk ) );      break;
	case '*':
		m_stack.append( Task( Reply::kMultiBulk ) ); break;
	default:
		break;
	}
}


MYXLIB_INLINE void Parser::readUnsafeString( const QString& value )
{
	if ( tos().m_reply.type() == Reply::kMultiBulk )
	{
		tos().m_count = value.toInt();
	}
	else
	{
		tos().m_reply.value() = value;
	}

	descend();
}


MYXLIB_INLINE void Parser::readSafeString( const QByteArray& value )
{
	tos().m_reply.value() = value;
	descend();
}


MYXLIB_INLINE void Parser::descend()
{
	while ( true )
	{
		auto& task = tos();
		if ( ( task.m_reply.type() == Reply::kMultiBulk ) &&
		     ( task.m_reply.value().toList().count() < task.m_count ) )
		{
			return;
		}

		if ( m_stack.count() == 1 )
		{
			if ( task.m_reply.type() == Reply::kMultiBulk )
			{
				Q_EMIT replyMultiBulk( m_stack.takeLast().m_reply );
			}
			else
			{
				Q_EMIT reply( m_stack.takeLast().m_reply );
			}
			return;
		}

		auto r = m_stack.takeLast().m_reply;

		Task t = m_stack.takeLast();
		auto l = t.m_reply.value().toList();
		l.append( QVariant::fromValue( r ) );
		t.m_reply.value().setValue( l );
		m_stack.append( t );
	}
} // Parser::descend

} // namespace redis

} // namespace myx

#endif // ifndef MYX_REDIS_PARSER_INL_HPP_
